-- define AceAddon
XengaInfo = LibStub('AceAddon-3.0'):NewAddon('XengaInfo', 'AceConsole-3.0', 'AceEvent-3.0');

-- variables
local xi_s = {
	char_class= '',
	char_lvl = 0,
	char_race = '',
	money_last = 0,
	money_start = 0,
	time_session_start = time(),
	first_load = true,
	current_expansion = 0,
	current_level_cap = 0
}

-- set default values for DB
local defaults = {
	profile = {
		enabled = true
	}
}

-- Ace options table
xi_options = {
	name = "Xenga Info",
	handler = XengaInfo,
	type = 'group',
	args = {
		enable = {
			name = 'Enable',
			desc = 'Enables / disables the addon',
			type = 'toggle',
			get = function(info) return XengaInfo.db.profile.enabled end,
			set = 'set_visibility'
		},
		reset_pos = {
			name = 'Reset position',
			desc = 'Resets the position of the addon. Useful if it has somehow moved off of your screen.',
			type = 'execute',
			func = 'reset_position'
		}
	}
}

-- reserved functions

function XengaInfo:OnDisable()
	-- not quite sure when this gets triggered
end

function XengaInfo:OnEnable()
	-- not quite sure when this gets triggered
end

function XengaInfo:OnInitialize()
	-- register options table
	LibStub('AceConfig-3.0'):RegisterOptionsTable('XengaInfo', xi_options);

	-- add to Bliz options
	self.options_frame = LibStub('AceConfigDialog-3.0'):AddToBlizOptions('XengaInfo', 'XengaInfo');

	-- register slash commands
	self:RegisterChatCommand('xi', 'chat_command_handler');
	self:RegisterChatCommand('xengainfo', 'chat_command_handler');

	-- get saved variables
	self.db = LibStub('AceDB-3.0'):New('xi_db', defaults, true);

	-- register event handlers
	self:set_visibility(nil, self.db.profile.enabled);
end


-- addon functions

-- chat command handler
function XengaInfo:chat_command_handler(input)
	if not input or input:trim() == '' then
		InterfaceOptionsFrame_OpenToCategory(self.options_frame);
	else
		LibStub('AceConfigCmd-3.0'):HandleCommand('xi', 'xengainfo', input);
	end
end

-- gets the level cap for the players expansion
function XengaInfo:get_level_cap()
	xi_s.current_expansion = GetAccountExpansionLevel();
	local exp_table = {
		[0] = 60,
		[1] = 70,
		[2] = 80,
		[3] = 85,
		[4] = 90,
		[5] = 100,
		[6] = 110
	}
	
	if exp_table[xi_s.current_expansion] then
		xi_s.current_level_cap = exp_table[xi_s.current_expansion];
	else
		xi_s.current_level_cap = 999;
	end
end

-- formats numerical money into a graphical string
function XengaInfo:format_money_string(money)
	local gold_str, silver_str, copper_str;
	local gold = floor(money / (COPPER_PER_SILVER * SILVER_PER_GOLD));
	local silver = floor((money - (gold * COPPER_PER_SILVER * SILVER_PER_GOLD)) / COPPER_PER_SILVER);
	local copper = mod(money, COPPER_PER_SILVER);

	if ( ENABLE_COLORBLIND_MODE == "1" ) then
		gold_str = self:format_number(gold)..GOLD_AMOUNT_SYMBOL;
		silver_str = silver .. SILVER_AMOUNT_SYMBOL;
		copper_str = copper .. COPPER_AMOUNT_SYMBOL;
	else
		gold_str =   format("%s\124TInterface\\MoneyFrame\\UI-GoldIcon:%d:%d:2:0\124t", self:format_number(gold), 0, 0);
		silver_str = format("%02d\124TInterface\\MoneyFrame\\UI-SilverIcon:%d:%d:2:0\124t", silver, 0, 0);
		copper_str = format("%02d\124TInterface\\MoneyFrame\\UI-CopperIcon:%d:%d:2:0\124t", copper, 0, 0);
	end

	local money_str = "";
	local separator = " ";

	money_str = gold_str .. separator .. silver_str .. separator .. copper_str;

	return money_str;
end

-- adds commas to numbers
function XengaInfo:format_number(n) -- credit http://richard.warburton.it
	local left, num, right = string.match(n,'^([^%d]*%d)(%d*)(.-)$');
	return left .. (num:reverse():gsub('(%d%d%d)', '%1, '):reverse()) .. right;
end

-- sets initial data
function XengaInfo:initial_player_load()
	if xi_s.first_load then
		self:get_level_cap();
		self:update_lvl_race_class_text(UnitLevel('player'));

		self:update_pvp_text();
		self:update_ilvl_text();

		self:update_zone_text();

		self:update_xp_text();

		xi_s.money_start = GetMoney();
		xi_s.money_last = xi_s.money_start;
		self:update_money_text();

		xi_s.first_load = false;
	end
end

-- registers events
function XengaInfo:register_events()
	self:RegisterEvent('PLAYER_ENTERING_WORLD', 'initial_player_load');
	self:RegisterEvent('PLAYER_EQUIPMENT_CHANGED', 'update_ilvl_text');
	self:RegisterEvent('PLAYER_LEVEL_UP', function(event, lvl) self:update_lvl_race_class_text(lvl) end);
	self:RegisterEvent('PLAYER_MONEY', 'update_money_text');
	self:RegisterEvent('PLAYER_PVP_KILLS_CHANGED', 'update_pvp_text');
	self:RegisterEvent('PLAYER_XP_UPDATE', 'update_xp_text');
	self:RegisterEvent('UNIT_INVENTORY_CHANGED', 'update_ilvl_text');
	self:RegisterEvent('WORLD_MAP_UPDATE', 'update_zone_text');
end

-- resets position of main frame
function XengaInfo:reset_position()
	xi_frame:ClearAllPoints();
	xi_frame:SetPoint('CENTER', 0, 0);
end

-- rounds a number
function XengaInfo:round(num, idp)
	local mult = 10^(idp or 0);
	return math.floor(num * mult + 0.5) / mult;
end

-- sets frame visibility
function XengaInfo:set_visibility(info, val)
	if val then
		self.db.profile.enabled = true;
		self:register_events();
		xi_frame:Show();
	else
		self.db.profile.enabled = false;
		self:unregister_events();
		xi_frame:Hide();
	end
end

-- unregisters events
function XengaInfo:unregister_events()
	self:UnregisterEvent('PLAYER_ENTERING_WORLD');
	self:UnregisterEvent('PLAYER_EQUIPMENT_CHANGED');
	self:UnregisterEvent('PLAYER_LEVEL_UP');
	self:UnregisterEvent('PLAYER_MONEY');
	self:UnregisterEvent('PLAYER_PVP_KILLS_CHANGED');
	self:UnregisterEvent('PLAYER_XP_UPDATE');
	self:UnregisterEvent('UNIT_INVENTORY_CHANGED');
	self:UnregisterEvent('WORLD_MAP_UPDATE');
end

-- update ivl text
function XengaInfo:update_ilvl_text()
	local o, e = GetAverageItemLevel();
	xi_char_item_level_text:SetText('iLvl: ' .. floor(e) .. '/' .. floor(o));
end

-- sets character level/race/class
function XengaInfo:update_lvl_race_class_text(level)
	if xi_s.char_class == '' then
		xi_s.char_class = UnitClass("player");
	end
	if xi_s.char_race == '' then
		xi_s.char_race = UnitRace("player");
	end

	xi_frame_frame_header_text:SetText('Lvl ' .. level .. ' ' .. xi_s.char_race .. ' ' .. xi_s.char_class);
end

-- updates money counters
function XengaInfo:update_money_text()
	local current_money = GetMoney();

	-- update session money frame
	local session_money = current_money - xi_s.money_start;
	if session_money < 0 then
		session_money = session_money * -1
		xi_money_session_text:SetTextColor(1.0, 0, 0);
	else
		xi_money_session_text:SetTextColor(1.0, 1.0, 1.0);
	end

	xi_money_current_text:SetText('Current: ' .. self:format_money_string(current_money));
	xi_money_session_text:SetText('Session: ' .. self:format_money_string(session_money));

	-- sets money for next event
	xi_s.money_last = money;

	-- adjust widths to they line up
	local current_w, session_w = xi_money_current_text:GetWidth(), xi_money_session_text:GetWidth();
	if current_w > session_w then
		xi_money_session_text:SetWidth(current_w);
	else
		xi_money_current_text:SetWidth(session_w);
	end
end

-- updates ping and time played
function XengaInfo:update_pandt_text()
	local down, up, ping = GetNetStats();
	xi_pandt_ping_text:SetText('Ping: ' .. ping);

	local time = (time() - xi_s.time_session_start);
 	local minutes, hours = 00, 00;

	minutes = mod(floor(time / 60), 60);
	hours = mod(floor(time / 3600), 3600);

	if( minutes < 10 ) then
		minutes = '0' .. minutes;
	end
	if( hours < 10 ) then
		hours = '0' .. hours;
	end

	xi_pandt_time_text:SetText('Session Time: ' .. hours .. ':' .. minutes);
end

-- update honor kills
function XengaInfo:update_pvp_text()
	local hk, dk, highestRank = GetPVPLifetimeStats();
	xi_char_honor_kills_text:SetText('HK: ' .. self:format_number(hk));
end

-- updates tnl/session xp text
function XengaInfo:update_xp_text()
	if ( UnitLevel('player') < xi_s.current_level_cap ) then
		--update tnl xp
		local tnl_xp = UnitXPMax('player') - UnitXP('player');
		local tnl_xp_percent = self:round(100-((UnitXP('player') / UnitXPMax('player')) * 100), 2);

		xi_xp_tnl_text:SetText('XP tnl: ' .. self:format_number(tnl_xp) .. ' (' .. tnl_xp_percent .. '%)');

		--update rested xp
		local rest_xp = GetXPExhaustion();
		if rest_xp == nil then
			rest_xp = '0';
		else
			rest_xp = self:round((rest_xp / 2), 2);
		end

		xi_xp_rested_text:SetText('Rest XP: ' .. self:format_number(rest_xp));

		-- adjust widths so they line up
		local tnl_w, rest_w = xi_xp_tnl_text:GetWidth(), xi_xp_rested_text:GetWidth();
		if tnl_w > rest_w then
			xi_xp_rested_text:SetWidth(tnl_w);
		else
			xi_xp_tnl_text:SetWidth(rest_w);
		end
	else
		xi_xp:Hide();
		xi_frame:SetHeight(xi_frame:GetHeight() - 40);
		xi_money:SetPoint('TOPLEFT', 'xi_zone', 'BOTTOMLEFT', 0, 0);
		xi_money:SetPoint('BOTTOMRIGHT', 'xi_zone', 'BOTTOMRIGHT', 0, -40);
	end
end

-- updates zone/sub zone text
function XengaInfo:update_zone_text()
	xi_zone_main_zone_text:SetText(GetRealZoneText());
	if GetRealZoneText() ~= GetMinimapZoneText() then
		xi_zone_sub_zone_text:SetText('"' .. GetMinimapZoneText() .. '"');
	else
		xi_zone_sub_zone_text:SetText("-");
	end
end
